@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -23,31 +23,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
 #include <libgda/gda-column.h>
 #include <string.h>
 #include "gda-marshal.h"
+#include <libgda/gda-attributes-manager.h>
+#include <libgda/gda-custom-marshal.h>
 
 #define PARENT_TYPE G_TYPE_OBJECT
 
 struct _GdaColumnPrivate {
 	gint         defined_size;
 	gchar       *id;
-	gchar       *name;
-	gchar       *title;
-
-	gchar       *table;
-	gchar       *caption;
-	gint         scale;
 
 	gchar       *dbms_type;
 	GType        g_type;
 
 	gboolean     allow_null;
-	gboolean     primary_key;
-	gboolean     unique_key;
-	gchar       *references;
 
 	gboolean     auto_increment;
 	glong        auto_increment_start;
@@ -86,6 +77,7 @@ enum
 };
 
 static GObjectClass *parent_class = NULL;
+GdaAttributesManager *gda_column_attributes_manager;
 
 static void
 gda_column_class_init (GdaColumnClass *klass)
@@ -95,8 +87,15 @@ gda_column_class_init (GdaColumnClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 	
 	/* signals */
+	/**
+	 * GdaColumn::name-changed
+	 * @column: the #GdaColumn object
+	 * @old_name: the column's previous name
+	 *
+	 * Gets emitted whenever @column's name has been changed
+	 */
 	gda_column_signals[NAME_CHANGED] =
-		g_signal_new ("name_changed",
+		g_signal_new ("name-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GdaColumnClass, name_changed),
@@ -104,45 +103,50 @@ gda_column_class_init (GdaColumnClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE,
 			      1, G_TYPE_STRING);
+	/**
+	 * GdaColumn::g-type-changed
+	 * @column: the #GdaColumn object
+	 * @old_type: the column's previous type
+	 * @new_type: the column's new type
+	 *
+	 * Gets emitted whenever @column's type has been changed
+	 */
 	gda_column_signals[GDA_TYPE_CHANGED] =
-		g_signal_new ("g_type_changed",
+		g_signal_new ("g-type-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GdaColumnClass, g_type_changed),
 			      NULL, NULL,
-			      gda_marshal_VOID__INT_INT,
+			      _gda_marshal_VOID__GTYPE_GTYPE,
 			      G_TYPE_NONE,
-			      2, G_TYPE_INT, G_TYPE_INT);
+			      2, G_TYPE_GTYPE, G_TYPE_GTYPE);
 
 	/* properties */
         object_class->set_property = gda_column_set_property;
         object_class->get_property = gda_column_get_property;
 
         g_object_class_install_property (object_class, PROP_ID,
-                                         g_param_spec_string ("id", NULL, NULL,
+                                         g_param_spec_string ("id", NULL, 
+							      "Column's Id (warning: the column's ID is not "
+							      "guaranteed to be unique in a GdaDataModel)",
                                                               NULL, G_PARAM_WRITABLE | G_PARAM_READABLE));
 
 	object_class->finalize = gda_column_finalize;
+
+	/* extra */
+	gda_column_attributes_manager = gda_attributes_manager_new (TRUE, NULL, NULL);
 }
 
 static void
-gda_column_init (GdaColumn *column, GdaColumnClass *klass)
+gda_column_init (GdaColumn *column, G_GNUC_UNUSED GdaColumnClass *klass)
 {
 	g_return_if_fail (GDA_IS_COLUMN (column));
 	
 	column->priv = g_new0 (GdaColumnPrivate, 1);
 	column->priv->defined_size = 0;
 	column->priv->id = NULL;
-	column->priv->name = NULL;
-	column->priv->table = NULL;
-	column->priv->title = NULL;
-	column->priv->caption = NULL;
-	column->priv->scale = 0;
 	column->priv->g_type = G_TYPE_INVALID;
 	column->priv->allow_null = TRUE;
-	column->priv->primary_key = FALSE;
-	column->priv->unique_key = FALSE;
-	column->priv->references = NULL;
 	column->priv->auto_increment = FALSE;
 	column->priv->auto_increment_start = 0;
 	column->priv->auto_increment_step = 0;
@@ -161,12 +165,8 @@ gda_column_finalize (GObject *object)
 		if (column->priv->default_value) 
 			gda_value_free (column->priv->default_value);
 	
-		g_free (column->priv->table);
 		g_free (column->priv->id);
-		g_free (column->priv->name);
-		g_free (column->priv->title);
-		g_free (column->priv->caption);
-		g_free (column->priv->references);
+		g_free (column->priv->dbms_type);
 
 		g_free (column->priv);
 		column->priv = NULL;
@@ -181,6 +181,7 @@ gda_column_get_type (void)
 	static GType type = 0;
 	
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaColumnClass),
 			(GBaseInitFunc) NULL,
@@ -190,9 +191,13 @@ gda_column_get_type (void)
 			NULL,
 			sizeof (GdaColumn),
 			0,
-			(GInstanceInitFunc) gda_column_init
+			(GInstanceInitFunc) gda_column_init,
+			0
 		};
-		type = g_type_register_static (PARENT_TYPE, "GdaColumn", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (PARENT_TYPE, "GdaColumn", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	
 	return type;
@@ -217,7 +222,8 @@ gda_column_set_property (GObject *object,
 				col->priv->id = NULL;
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -237,13 +243,14 @@ gda_column_get_property (GObject *object,
 			g_value_set_string (value, col->priv->id);
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
 
 /**
- * gda_column_new
+ * gda_column_new:
  *
  * Returns: a newly allocated #GdaColumn object.
  */
@@ -258,11 +265,12 @@ gda_column_new (void)
 }
 
 /**
- * gda_column_copy 	 
+ * gda_column_copy:
  * @column: column to get a copy from. 	 
  * 	 
- * Creates a new #GdaColumn object from an existing one. 	 
- * Returns: a newly allocated #GdaColumn with a copy of the data 	 
+ * Creates a new #GdaColumn object from an existing one.
+ *
+ * Returns: (transfer full): a newly allocated #GdaColumn with a copy of the data 	 
  * in @column. 	 
  */ 	 
 GdaColumn * 	 
@@ -276,60 +284,21 @@ gda_column_copy (GdaColumn *column)
 	column_copy->priv->defined_size = column->priv->defined_size;
 	if (column->priv->id)
 		column_copy->priv->id = g_strdup (column->priv->id);
-	if (column->priv->name)
-		column_copy->priv->name = g_strdup (column->priv->name);
-	if (column->priv->title)
-		column_copy->priv->title = g_strdup (column->priv->title);
-	if (column->priv->table)
-		column_copy->priv->table = g_strdup (column->priv->table);
-	if (column->priv->caption)
-		column_copy->priv->caption = g_strdup (column->priv->caption);
-	column_copy->priv->scale = column->priv->scale;
 	column_copy->priv->g_type = column->priv->g_type;
 	column_copy->priv->allow_null = column->priv->allow_null;
-	column_copy->priv->primary_key = column->priv->primary_key;
-	column_copy->priv->unique_key = column->priv->unique_key;
-	if (column->priv->references)
-		column_copy->priv->references = g_strdup (column->priv->references);
 	column_copy->priv->auto_increment = column->priv->auto_increment;
 	column_copy->priv->auto_increment_start = column->priv->auto_increment_start;
 	column_copy->priv->auto_increment_step = column->priv->auto_increment_step;
 	column_copy->priv->position = column->priv->position;
 	if (column->priv->default_value)
 		column_copy->priv->default_value = gda_value_copy (column->priv->default_value);
-	
+	gda_attributes_manager_copy (gda_column_attributes_manager, (gpointer) column, gda_column_attributes_manager, (gpointer) column_copy);
+
 	return column_copy; 	 
 }
 
 /**
- * gda_column_get_defined_size
- * @column: a @GdaColumn.
- *
- * Returns: the defined size of @column.
- */
-glong
-gda_column_get_defined_size (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), 0);
-	return column->priv->defined_size;
-}
-
-/**
- * gda_column_set_defined_size
- * @column: a #GdaColumn.
- * @size: the defined size we want to set.
- *
- * Sets the defined size of a #GdaColumn.
- */
-void
-gda_column_set_defined_size (GdaColumn *column, glong size)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-	column->priv->defined_size = size;
-}
-
-/**
- * gda_column_get_name
+ * gda_column_get_name:
  * @column: a #GdaColumn.
  *
  * Returns: the name of @column.
@@ -337,12 +306,17 @@ gda_column_set_defined_size (GdaColumn *column, glong size)
 const gchar *
 gda_column_get_name (GdaColumn *column)
 {
+	const GValue *cvalue;
 	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return (const gchar *) column->priv->name;
+	cvalue = gda_column_get_attribute (column, GDA_ATTRIBUTE_NAME);
+	if (cvalue)
+		return g_value_get_string (cvalue);
+	else
+		return NULL;
 }
 
 /**
- * gda_column_set_name
+ * gda_column_set_name:
  * @column: a #GdaColumn.
  * @name: the new name of @column.
  *
@@ -352,148 +326,71 @@ void
 gda_column_set_name (GdaColumn *column, const gchar *name)
 {
 	gchar *old_name = NULL;
+	GValue *value = NULL;
 
 	g_return_if_fail (GDA_IS_COLUMN (column));
 
-	if (column->priv->name) {
-		old_name = column->priv->name;
-		column->priv->name = NULL;
-	}
+	old_name = (gchar *) gda_column_get_name (column);
+	if (old_name)
+		old_name = g_strdup (old_name);
 
 	if (name)
-		column->priv->name = g_strdup (name);
+		g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), name);
+	gda_column_set_attribute_static (column, GDA_ATTRIBUTE_NAME, value);
+	if (value)
+		gda_value_free (value);
 
 	g_signal_emit (G_OBJECT (column),
 		       gda_column_signals[NAME_CHANGED],
 		       0, old_name);
 
-	if (old_name)
-		g_free (old_name);
+	g_free (old_name);
 }
 
 /**
- * gda_column_get_title
+ * gda_column_get_description:
  * @column: a #GdaColumn.
  *
- * Returns: the column's title
+ * Returns: the column's description, in any
  */
 const gchar *
-gda_column_get_title (GdaColumn *column)
+gda_column_get_description (GdaColumn *column)
 {
+	const GValue *cvalue;
 	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return column->priv->title;;
+	cvalue = gda_column_get_attribute (column, GDA_ATTRIBUTE_DESCRIPTION);
+	if (cvalue)
+		return g_value_get_string (cvalue);
+	else
+		return NULL;
 }
 
 /**
- * gda_column_set_title
+ * gda_column_set_description:
  * @column: a #GdaColumn.
  * @title: title name.
  *
- * Sets the column's title
+ * Sets the column's description
  */
 void
-gda_column_set_title (GdaColumn *column, const gchar *title)
+gda_column_set_description (GdaColumn *column, const gchar *descr)
 {
+	GValue *value = NULL;
+
 	g_return_if_fail (GDA_IS_COLUMN (column));
 
-	if (column->priv->title != NULL)
-		g_free (column->priv->title);
-	column->priv->title = g_strdup (title);
-}
-
-
-/**
- * gda_column_get_table
- * @column: a #GdaColumn.
- *
- * Returns: the name of the table to which this column belongs.
- */
-const gchar *
-gda_column_get_table (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return column->priv->table;;
+	if (descr)
+		g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), descr);
+	gda_column_set_attribute_static (column, GDA_ATTRIBUTE_DESCRIPTION, value);
+	if (value)
+		gda_value_free (value);
 }
 
 /**
- * gda_column_set_table
- * @column: a #GdaColumn.
- * @table: table name.
- *
- * Sets the name of the table to which the given column belongs.
- */
-void
-gda_column_set_table (GdaColumn *column, const gchar *table)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-
-	if (column->priv->table != NULL)
-		g_free (column->priv->table);
-	column->priv->table = g_strdup (table);
-}
-
-/**
- * gda_column_get_caption
+ * gda_column_get_dbms_type:
  * @column: a #GdaColumn.
  *
- * Returns: @column's caption.
- */
-const gchar *
-gda_column_get_caption (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return (const gchar *) column->priv->caption;
-}
-
-/**
- * gda_column_set_caption
- * @column: a #GdaColumn.
- * @caption: caption.
- *
- * Sets @column's @caption.
- */
-void
-gda_column_set_caption (GdaColumn *column, const gchar *caption)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-
-	if (column->priv->caption)
-		g_free (column->priv->caption);
-	column->priv->caption = g_strdup (caption);
-}
-
-/**
- * gda_column_get_scale
- * @column: a #GdaColumn.
- *
- * Returns: the number of decimals of @column.
- */
-glong
-gda_column_get_scale (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), 0);
-	return column->priv->scale;
-}
-
-/**
- * gda_column_set_scale
- * @column: a #GdaColumn.
- * @scale: number of decimals.
- *
- * Sets the scale of @column to @scale.
- */
-void
-gda_column_set_scale (GdaColumn *column, glong scale)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-	column->priv->scale = scale;
-}
-
-/**
- * gda_column_get_dbms_type
- * @column: a #GdaColumn.
- *
- * Returns: the dbms_type of @column.
+ * Returns: the database type of @column.
  */
 const gchar*
 gda_column_get_dbms_type (GdaColumn *column)
@@ -502,6 +399,13 @@ gda_column_get_dbms_type (GdaColumn *column)
 	return (const gchar *) column->priv->dbms_type;
 }
 
+/**
+ * gda_column_set_dbms_type:
+ * @column: a #GdaColumn
+ * @dbms_type: a string
+ *
+ * Defines @column's database type
+ */
 void
 gda_column_set_dbms_type (GdaColumn *column, const gchar *dbms_type)
 {
@@ -517,7 +421,7 @@ gda_column_set_dbms_type (GdaColumn *column, const gchar *dbms_type)
 }
 
 /**
- * gda_column_get_g_type
+ * gda_column_get_g_type:
  * @column: a #GdaColumn.
  *
  * Returns: the type of @column.
@@ -530,7 +434,7 @@ gda_column_get_g_type (GdaColumn *column)
 }
 
 /**
- * gda_column_set_g_type
+ * gda_column_set_g_type:
  * @column: a #GdaColumn.
  * @type: the new type of @column.
  *
@@ -552,7 +456,7 @@ gda_column_set_g_type (GdaColumn *column, GType type)
 }
 
 /**
- * gda_column_get_allow_null
+ * gda_column_get_allow_null:
  * @column: a #GdaColumn.
  *
  * Gets the 'allow null' flag of the given column.
@@ -567,7 +471,7 @@ gda_column_get_allow_null (GdaColumn *column)
 }
 
 /**
- * gda_column_set_allow_null
+ * gda_column_set_allow_null:
  * @column: a #GdaColumn.
  * @allow: whether the given column should allows null values or not.
  *
@@ -581,98 +485,7 @@ gda_column_set_allow_null (GdaColumn *column, gboolean allow)
 }
 
 /**
- * gda_column_get_primary_key
- * @column: a #GdaColumn.
- *
- * Returns: whether if the given column is a primary key (%TRUE or %FALSE).
- */
-gboolean
-gda_column_get_primary_key (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), FALSE);
-	return column->priv->primary_key;
-}
-
-/**
- * gda_column_set_primary_key
- * @column: a #GdaColumn.
- * @pk: whether if the given column should be a primary key.
- *
- * Sets the 'primary key' flag of the given column.
- */
-void
-gda_column_set_primary_key (GdaColumn *column, gboolean pk)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-	column->priv->primary_key = pk;
-}
-
-/**
- * gda_column_get_unique_key
- * @column: a #GdaColumn.
- *
- * Returns: whether if the given column is an unique key (%TRUE or %FALSE).
- */
-gboolean
-gda_column_get_unique_key (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), FALSE);
-	return column->priv->unique_key;
-}
-
-/**
- * gda_column_set_unique_key
- * @column: a #GdaColumn.
- * @uk: whether if the given column should be an unique key.
- *
- * Sets the 'unique key' flag of the given column.
- */
-void
-gda_column_set_unique_key (GdaColumn *column, gboolean uk)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-	column->priv->unique_key = uk;
-}
-
-/**
- * gda_column_get_references
- * @column: a #GdaColumn.
- *
- * Reference is returned in tablename.fieldname format. Do not free
- * this variable, it is used internally within GdaColumn.
- *
- * Returns: @column's references.
- */
-const gchar *
-gda_column_get_references (GdaColumn *column)
-{
-	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return (const gchar *) column->priv->references;
-}
-
-/**
- * gda_column_set_references
- * @column: a #GdaColumn.
- * @ref: references.
- *
- * Sets @column's @references.
- */
-void
-gda_column_set_references (GdaColumn *column, const gchar *ref)
-{
-	g_return_if_fail (GDA_IS_COLUMN (column));
-
-	if (column->priv->references != NULL) {
-		g_free (column->priv->references);
-		column->priv->references = NULL;
-	}
-
-	if (ref)
-		column->priv->references = g_strdup (ref);
-}
-
-/**
- * gda_column_get_auto_increment
+ * gda_column_get_auto_increment:
  * @column: a #GdaColumn.
  *
  * Returns: whether the given column is an auto incremented one (%TRUE or %FALSE).
@@ -685,7 +498,7 @@ gda_column_get_auto_increment (GdaColumn *column)
 }
 
 /**
- * gda_column_set_auto_increment
+ * gda_column_set_auto_increment:
  * @column: a #GdaColumn.
  * @is_auto: auto increment status.
  *
@@ -700,7 +513,7 @@ gda_column_set_auto_increment (GdaColumn *column,
 }
 
 /**
- * gda_column_get_position
+ * gda_column_get_position:
  * @column: a #GdaColumn.
  *
  * Returns: the position of the column refer to in the
@@ -714,7 +527,7 @@ gda_column_get_position (GdaColumn *column)
 }
 
 /**
- * gda_column_set_position
+ * gda_column_set_position:
  * @column: a #GdaColumn.
  * @position: the wanted position of the column in the containing data model.
  *
@@ -730,7 +543,7 @@ gda_column_set_position (GdaColumn *column, gint position)
 
 
 /**
- * gda_column_get_default_value
+ * gda_column_get_default_value:
  * @column: a #GdaColumn.
  *
  * Returns: @column's default value, as a #GValue object.
@@ -743,7 +556,7 @@ gda_column_get_default_value (GdaColumn *column)
 }
 
 /**
- * gda_column_set_default_value
+ * gda_column_set_default_value:
  * @column: a #GdaColumn.
  * @default_value: default #GValue for the column
  *
@@ -759,3 +572,63 @@ gda_column_set_default_value (GdaColumn *column, const GValue *default_value)
 		gda_value_free (column->priv->default_value);
 	column->priv->default_value = gda_value_copy ( (GValue*)default_value);
 }
+
+/**
+ * gda_column_get_attribute:
+ * @column: a #GdaColumn
+ * @attribute: attribute name as a string
+ *
+ * Get the value associated to a named attribute.
+ *
+ * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-40-Attributes-manager.synopsis">this section</link>.
+ *
+ * Returns: a read-only #GValue, or %NULL if not attribute named @attribute has been set for @column
+ */
+const GValue *
+gda_column_get_attribute (GdaColumn *column, const gchar *attribute)
+{
+	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
+	return gda_attributes_manager_get (gda_column_attributes_manager, column, attribute);
+}
+
+/**
+ * gda_column_set_attribute:
+ * @column: a #GdaColumn
+ * @attribute: attribute name as a static string
+ * @value: a #GValue, or %NULL
+ * @destroy: a function to be called when @attribute is not needed anymore, or %NULL
+ *
+ * Set the value associated to a named attribute. The @attribute string is 'stolen' by this method, and
+ * the memory it uses will be freed using the @destroy function when no longer needed (if @destroy is %NULL,
+ * then the string will not be freed at all).
+ *
+ * Attributes can have any name, but Libgda proposes some default names, 
+ * see <link linkend="libgda-40-Attributes-manager.synopsis">this section</link>.
+ *
+ * If there is already an attribute named @attribute set, then its value is replaced with the new value (@value is
+ * copied), except if @value is %NULL, in which case the attribute is removed.
+ *
+ * For example one would use it as:
+ *
+ * <code>
+ * gda_column_set_attribute (holder, g_strdup (my_attribute), g_free, my_value);
+ * gda_column_set_attribute (holder, GDA_ATTRIBUTE_NAME, NULL, my_value);
+ * </code>
+ *
+ * Note: this method does not modify in any way the contents of the data model for which @column is a column (nor
+ * does it modify the table definition of the tables used by a SELECT statement is the model was created from a
+ * SELECT statement).
+ */
+void
+gda_column_set_attribute (GdaColumn *column, const gchar *attribute, const GValue *value, GDestroyNotify destroy)
+{
+	const GValue *cvalue;
+	g_return_if_fail (GDA_IS_COLUMN (column));
+
+	cvalue = gda_attributes_manager_get (gda_column_attributes_manager, column, attribute);
+	if ((value && cvalue && !gda_value_differ (cvalue, value)) ||
+	    (!value && !cvalue))
+		return;
+
+	gda_attributes_manager_set_full (gda_column_attributes_manager, column, attribute, value, destroy);
+}
