@@ -114,12 +114,12 @@ test_cnc_setup_connection (const gchar *provider, const gchar *dbname, GError **
 		GdaServerOperation *op;
 
 		db_quark_list = gda_quark_list_new_from_string (db_params);
-		op = gda_prepare_drop_database (prov_info->id, dbname, NULL);
+		op = gda_server_operation_prepare_drop_database (prov_info->id, dbname, NULL);
 		gda_quark_list_foreach (db_quark_list, (GHFunc) db_create_quark_foreach_func, op);
-		gda_perform_drop_database (NULL, op, NULL);
+		gda_server_operation_perform_drop_database (op, NULL, NULL);
 		g_object_unref (op);
 
-		op = gda_prepare_create_database (prov_info->id, dbname, NULL);
+		op = gda_server_operation_prepare_create_database (prov_info->id, dbname, NULL);
 		gda_quark_list_foreach (db_quark_list, (GHFunc) db_create_quark_foreach_func, op);
 		if (!gda_perform_create_database (NULL, op, error)) 
 			goto out;
@@ -294,13 +294,13 @@ test_cnc_clean_connection (GdaConnection *cnc, GError **error)
 		g_free (str);
 		g_assert (db_params);
 
-		op = gda_prepare_drop_database (prov_id, dbname, NULL);
+		op = gda_server_operation_prepare_drop_database (prov_id, dbname, NULL);
 		g_free (dbname);
 		db_quark_list = gda_quark_list_new_from_string (db_params);
 		gda_quark_list_foreach (db_quark_list, (GHFunc) db_drop_quark_foreach_func, op);
 		gda_quark_list_free (db_quark_list);
 
-		if (!gda_perform_drop_database (NULL, op, error)) 
+		if (!gda_server_operation_perform_drop_database (op, NULL, error))
 			retval = FALSE;
 		g_object_unref (op);
 	}
@@ -316,7 +316,8 @@ test_cnc_clean_connection (GdaConnection *cnc, GError **error)
 }
 
 gboolean
-test_cnc_setup_db_contents (GdaConnection *cnc, const gchar *data_file, GError **error)
+test_cnc_setup_db_contents (G_GNUC_UNUSED GdaConnection *cnc, G_GNUC_UNUSED const gchar *data_file,
+			    G_GNUC_UNUSED GError **error)
 {
 	TO_IMPLEMENT;
 	return FALSE;
@@ -328,7 +329,7 @@ test_cnc_setup_db_contents (GdaConnection *cnc, const gchar *data_file, GError *
 gboolean
 test_cnc_load_data_from_file (GdaConnection *cnc, const gchar *table, const gchar *full_file, GError **error)
 {
-	GdaStatement *stmt;
+	GdaStatement *stmt = NULL;
 	GdaSet *params = NULL;
 	GdaDataModel *import;
 	gint nrows, ncols, i;
